@@ -574,18 +574,18 @@ sub compile_into {
     my ($class, $package, $c_code, %opts) = @_;
     my $compile_params = ($class->full_debugging ?
                           <<'COMPILE_PARAMS_DEBUG' :
-    CCFLAGS => "-Wall -Wno-unused -Werror -save-temps",
+    CCFLAGS => "-Wall -Wno-compound-token-split-by-macro -Wno-unused -Werror -save-temps",
     OPTIMIZE => "-g",
     CLEAN_AFTER_BUILD => 0,
 COMPILE_PARAMS_DEBUG
                           <<'COMPILE_PARAMS_OPTIMIZED');
-    OPTIMIZE => "-g -O2",
+    OPTIMIZE => "-g -O2  -Wno-compound-token-split-by-macro",
 COMPILE_PARAMS_OPTIMIZED
 
-      my $openssl_params = sprintf('LIBS => "%s -lcrypto -lssl",',
+      my $openssl_params = sprintf("LIBS => '%s -lcrypto -lssl',",
                                    ($ENV{BUILD_OPENSSL_LDFLAGS} or ""));
     if ($ENV{BUILD_OPENSSL_CFLAGS}) {
-        $openssl_params .= qq' INC => "$ENV{BUILD_OPENSSL_CFLAGS}",';
+        $openssl_params .= " INC => '$ENV{BUILD_OPENSSL_CFLAGS}',";
     }
 
     my $version_params =
@@ -734,14 +734,16 @@ __END__
 
 =cut
 
-use Test::More "no_plan";
-use Test::Group;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test qw(errstack_empty_ok
                                 cannot_check_SV_leaks leaks_SVs_ok
                                 cannot_check_bytes_leaks leaks_bytes_ok);
 use Data::Dumper;
 
-test "synopsis" => sub {
+# We are going to compile some C code as part of the test suite:
+$ENV{$_} = $main::ENVorig{$_} for qw(BUILD_OPENSSL_CFLAGS BUILD_OPENSSL_LDFLAGS);
+
+subtest "synopsis" => sub {
     my $idiom = My::Tests::Below->pod_code_snippet("synopsis");
 
     my $some_c_code = <<"SOME_C_CODE";
@@ -756,7 +758,7 @@ SOME_C_CODE
     is($result, undef);
 };
 
-test 'perl_wrap() and perl_unwrap()' => sub {
+subtest 'perl_wrap() and perl_unwrap()' => sub {
     # Also doubles as a learning test for Inline
     use Crypt::OpenSSL::CA::Inline::C <<"C_TEST";
 SV* make_bogus_object(int value) {
@@ -799,7 +801,7 @@ C_TEST
     is($object, undef);
 };
 
-test '$c_boilerplate: char0_value()' => sub {
+subtest '$c_boilerplate: char0_value()' => sub {
     { package Char0Test; use Crypt::OpenSSL::CA::Inline::C <<"CHAR0_TEST"; }
 
 static
@@ -814,8 +816,8 @@ CHAR0_TEST
        "", "char0_value shall not SEGV on undef");
 };
 
-skip_next_test "Devel::Leak needed" if cannot_check_SV_leaks;
-test 'OO and reference counting using $c_boilerplate' => sub {
+subtest 'OO and reference counting using $c_boilerplate' => sub {
+    skip_all "Devel::Leak needed" if cannot_check_SV_leaks;
     # This also doubles as a learning test (for OO style)
     { package Foo; use Crypt::OpenSSL::CA::Inline::C <<"C_TEST"; }
 static
@@ -892,7 +894,7 @@ void test_parse_serial_or_croak(char* serial) {
 C_TEST
 
 
-test "sslcroak()" => sub {
+subtest "sslcroak()" => sub {
     # Implementation lifted from Crypt::OpenSSL::CA so as
     # to sever the circular dependency in tests:
     sub Crypt::OpenSSL::CA::_sslcroak_callback {
@@ -940,7 +942,7 @@ test "sslcroak()" => sub {
     }
 };
 
-test "parse_RFC3280_time" => sub {
+subtest "parse_RFC3280_time" => sub {
     is(TestCRoutines::test_parse_RFC3280_time("20510103103442Z"), 1);
     is(TestCRoutines::test_parse_RFC3280_time("19510103103442Z"), 0);
     TestCRoutines::test_parse_RFC3280_time_or_croak("19510103103442Z");
@@ -950,8 +952,8 @@ test "parse_RFC3280_time" => sub {
     };
 };
 
-skip_next_test "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
-test "parse_RFC3280_time_or_croak memory leaks" => sub {
+subtest "parse_RFC3280_time_or_croak memory leaks" => sub {
+    skip_all "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
     leaks_bytes_ok {
         for(1..10000) {
             eval {
@@ -964,7 +966,7 @@ test "parse_RFC3280_time_or_croak memory leaks" => sub {
     };
 };
 
-test "parse_serial" => sub {
+subtest "parse_serial" => sub {
     TestCRoutines::test_parse_serial_or_croak("0xdeadbeef1234");
     pass;
     eval {
@@ -977,8 +979,8 @@ test "parse_serial" => sub {
     is(TestCRoutines::test_parse_serial("deadbeef1234"), 1);
 };
 
-skip_next_test "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
-test "parse_serial memory leaks" => sub {
+subtest "parse_serial memory leaks" => sub {
+    skip_all "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
     leaks_bytes_ok {
         for(1..10000) {
             TestCRoutines::test_parse_serial("0xdeadbeef1234");
@@ -987,6 +989,8 @@ test "parse_serial memory leaks" => sub {
     };
 };
 
+done_testing;
+
 =end internals
 
 =cut
