@@ -4,6 +4,9 @@ package Crypt::OpenSSL::CA::Test;
 
 use warnings;
 use strict;
+use Test2::API qw/context/;
+use Test2::Tools::Compare;
+use Test2::Tools::Basic;
 
 =head1 NAME
 
@@ -14,7 +17,7 @@ B<Crypt::OpenSSL::CA::Test> - Testing L<Crypt::OpenSSL::CA>
 =for My::Tests::Below "synopsis" begin
 
   use Crypt::OpenSSL::CA::Test qw(:default %test_der_DNs);
-  use Test::Group;
+  use Test2::V0;
 
   my $utf8 = Crypt::OpenSSL::CA::Test->test_simple_utf8();
 
@@ -23,18 +26,18 @@ B<Crypt::OpenSSL::CA::Test> - Testing L<Crypt::OpenSSL::CA>
   warn "Hello world";
   SCRIPT
 
-  skip_next_test "Devel::Leak needed" if cannot_check_SV_leaks;
-  test "leaky code" => sub {
+  subtest "leaky code" => sub {
+     skip_all "Devel::Leak needed" if cannot_check_SV_leaks;
      leaks_SVs_ok {
         # Do stuff
      }, -max => 6;
   };
 
-  skip_next_test "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
-  test "even leakier code" => sub {
+  subtest "even leakier code" => sub {
+     skip_all "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
      leaks_bytes_ok {
        # Do stuff
-     }, -max => 51200;
+     }, -max => 65536;
   };
 
 =for My::Tests::Below "synopsis" end
@@ -61,9 +64,6 @@ be exported upon request.
 
 =cut
 
-use Test::Builder;
-use Test::More;
-use Test::Group;
 use File::Path ();
 use File::Spec ();
 use File::Slurp ();
@@ -74,6 +74,7 @@ BEGIN {
     our @EXPORT =
         qw(openssl_path run_thru_openssl
            dumpasn1_available run_dumpasn1
+           like_bigint
            run_perl run_perl_ok run_perl_script run_perl_script_ok
            errstack_empty_ok
            certificate_looks_ok
@@ -170,6 +171,33 @@ sub run_dumpasn1 {
     return $out;
  }
 
+=head2 like_bigint ($text, $bigint)
+
+=head2 like_bigint ($text, $bigint, $testname)
+
+Like L<Test2::Tools::Compare/like>, but find a L<bigint> instead of a
+regex.
+
+Checks whether the bigint C<$bigint> can be found in C<$text> in a
+variety of formats (e.g. decimal or hex, with or without byte
+separators). Reports to the test system like C<like()> would.
+
+=cut
+
+sub like_bigint {
+    my ($text, $bigint, $testname) = @_;
+    $testname ||= "like_bigint";
+
+    my $context = context();
+
+    my $dec = $bigint->bstr;
+    my $hex = $bigint->to_hex;
+    my $hex_sep = join("[:.]", $bigint->to_hex =~ m/../g);
+    my $retval = like($text, qr/(?:$dec|$hex|$hex_sep)/i, $testname);
+
+    $context->release();
+    return $retval;
+}
 
 =head2 run_perl ($scripttext)
 
@@ -234,18 +262,22 @@ sub run_perl_script {
 }
 
 BEGIN { foreach my $functionname (qw(run_perl run_perl_script)) {
-  my $ok_wrapper = sub {
-    my ($code, $outref, $testname) = @_;
-    local $Test::Builder::Level = $Test::Builder::Level + 1;
-    $testname ||= $functionname;
-    my $out = __PACKAGE__->can($functionname)->($code);
-    $$outref = $out if ref($outref) eq "SCALAR";
-    my $retval = is($?, 0, $testname);
-    diag($out) if ! $retval;
-    return $retval;
-  };
-  no strict "refs";
-  *{"${functionname}_ok"} = $ok_wrapper;
+    my $ok_wrapper = sub {
+        my ($script_or_file, $outref, $testname) = @_;
+        $testname ||= $functionname;
+
+        my $context = context();
+        my $out = __PACKAGE__->can($functionname)->($script_or_file);
+        $$outref = $out if ref($outref) eq "SCALAR";
+        if ($?) {
+            return $context->fail_and_release($testname, "exited with code $?", $out);
+        } else {
+            return $context->pass_and_release($testname);
+        }
+    };
+
+    no strict "refs";
+    *{"${functionname}_ok"} = $ok_wrapper;
 }}
 
 =head2 _perl_cmdline ()
@@ -298,17 +330,19 @@ be run at the end of every test.
 =cut
 
 sub errstack_empty_ok {
-    local $Test::Builder::Level = $Test::Builder::Level + 1;
-    # Net::SSLeay provides an undocumented access to OpenSSL's ERR_
-    # API.  Fortunately, thanks to my newly acquired XS-Fu, reading
-    # the source was a piece of cake!
+    my $context = context();
+
+    my @ssleay_errors;
     require Net::SSLeay;
-    my $errcount = 0;
     while(my $error = Net::SSLeay::ERR_get_error()) {
-        diag(Net::SSLeay::ERR_error_string($error));
-        $errcount++;
+        push(@ssleay_errors, $error);
+    }
+    if (@ssleay_errors) {
+        return $context->fail_and_release("Errors found on OpenSSL's stack",
+                                          @ssleay_errors);
+    } else {
+        return $context->pass_and_release("No errors found on OpenSSL's stack");
     }
-    return is($errcount, 0, "number of errors found on OpenSSL's stack");
 }
 
 =head2 cannot_check_SV_leaks ()
@@ -344,7 +378,7 @@ Available named arguments are:
 
 =item I<< -name => $testname >>
 
-The name of the test, as in the second argument to L<Test::Builder/ok>.
+The name of the test, as in the second argument to L<Test2::V0/ok>.
 
 =item I<< -max => $threshold >>
 
@@ -358,12 +392,24 @@ needs a couple of SVs of its own.
 
 sub leaks_SVs_ok (&@) {
     my ($coderef, %args) = @_;
-    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    my $testname = $args{-name} || "leaks_SVs_ok";
+
+    my $context = context();
+
     require Devel::Leak;
     my $handle; my $count = Devel::Leak::NoteSV($handle);
     $coderef->();
-    my $consumed_SVs = Devel::Leak::CheckSV($handle) - $count;
-    cmp_ok($consumed_SVs, "<=", ($args{-max} || 6), ($args{-name} || "leaks_SVs_ok"));
+
+    my $leaked = Devel::Leak::CheckSV($handle) - $count;
+    my $tolerated = 6;
+
+    if ($leaked > $tolerated) {
+        return $context->fail_and_release(
+          $testname,
+          "Too many scalars leaked ($leaked, max $tolerated)");
+    }
+
+    return $context->pass_and_release($testname);
 }
 
 
@@ -382,11 +428,11 @@ Available named arguments are:
 
 =item I<< -name => $testname >>
 
-The name of the test, as in the second argument to L<Test::Builder/ok>.
+The name of the test, as in the second argument to L<Test2::V0/ok>.
 
 =item I<< -max => $threshold >>
 
-The minimum number of leaked bytes to look for.  The default is 51200.
+The minimum number of leaked bytes to look for.  The default is 65536.
 Setting this too low will trigger false positives, as Perl does some
 funky memory management eg in hash tables and that may cause jitter in
 the memory consumption as measured from malloc's point of view.
@@ -397,18 +443,25 @@ the memory consumption as measured from malloc's point of view.
 
 sub leaks_bytes_ok (&@) {
     my ($coderef, %args) = @_;
+    my $testname = $args{-name} || "leaks_byes_ok";
+
+    my $context = context();
+
     require Devel::Mallinfo;
-
     my $size_before = Devel::Mallinfo::mallinfo()->{uordblks} || 0;
-
     $coderef->();
-
     my $size_after = Devel::Mallinfo::mallinfo()->{uordblks} || 0;
 
-    my $consumption = $size_after - $size_before;
-    local $Test::Builder::Level = $Test::Builder::Level + 1;
-    cmp_ok($consumption, "<", ($args{-max} || 51200),
-          $args{-name} || "leaks_bytes_ok");
+    my $leaked = $size_after - $size_before;
+    my $tolerated = $args{-max} || 65536;
+
+    if ($leaked > $tolerated) {
+        return $context->fail_and_release(
+          $testname,
+          "Too many bytes leaked ($leaked, max $tolerated)");
+    }
+
+    return $context->pass_and_release($testname);
 }
 
 =head2 certificate_looks_ok ($pem_certificate)
@@ -425,19 +478,28 @@ sub certificate_looks_ok {
     my ($pem_certificate, $test_name) = @_;
 
     $test_name ||= "certificate_looks_ok";
-    test $test_name => sub {
-        my ($out, $err);
-        ($out, $err) =
-            run_thru_openssl($pem_certificate, qw(x509 -noout -text));
-        unless (is($?, 0, "openssl execution failed with code $?")) {
-            diag $err;
-            return;
-        }
-        unlike($out, qr/error/,
-             "openssl seemed to dislike the certificate");
-        like($out, qr/Certificate:/,
-             "openssl seemed not to be able to parse the certificate");
-    };
+
+    my $context = context();
+    my ($out, $err) =
+      run_thru_openssl($pem_certificate, qw(x509 -noout -text));
+    if ($? != 0) {
+        return $context->fail_and_release($test_name,
+                                          "openssl execution failed with code $?");
+    }
+    if ($out =~ m/error/) {
+        return $context->fail_and_release(
+            $test_name,
+            "openssl appears to have shown an error",
+           $out, $err);
+
+    } elsif ($out !~ m/Certificate:/) {
+        return $context->fail_and_release(
+            $test_name,
+            "openssl could not seem to parse the certificate",
+           $out, $err);
+    } else {
+        return $context->pass_and_release($test_name);
+    }
 }
 
 =head2 certificate_chain_ok ($pem_certificate, \@certchain )
@@ -453,21 +515,28 @@ passed as a reference.
 sub certificate_chain_ok {
     my ($cert, $certchain, $testname) = @_;
 
-    test (($testname || "certificate_chain_ok") => sub {
-        my $out = _run_openssl_verify($cert, $certchain, $testname);
-        return if ! defined $out; # Already failed
-        like($out, qr/OK/, "verify successful");
-        unlike($out, qr/error/, "no errors");
-    });
+    my $context = context();
+
+    my $out = _run_openssl_verify($cert, $certchain, $testname);
+    do { $context->release; return } if (! defined $out);  # Failed on previous line
+
+    if ($out =~ m/error/) {
+        return $context->fail_and_release($testname, "openssl found errors", $out);
+    } elsif ($out !~ m/OK/) {
+        return $context->fail_and_release($testname, "openssl did not verify OK", $out);
+    } else {
+        return $context->pass_and_release($testname);
+    }
 }
 
 sub _run_openssl_verify {
     my ($cert, $certchain, $testname) = @_;
 
-    # This is mostly a hack to get the test suite to
-    # work, but CA:FALSE certificates *really* should
-    # not be made part of a certification chain.
+    my $context = context();
 
+    # Filter out CA:FALSE certificates - mostly a hack to get the test
+    # suite to work; but such certs *really* should not be used in an
+    # actual certification chain.
     my @certchain = grep {
         my $out = run_thru_openssl($_, qw(x509 -noout -text));
         ( $out =~ m/CA:TRUE/ ) ? 1 : (warn(<<"WARNING"), 0);
@@ -475,14 +544,20 @@ ${$testname ? \"$testname: " : \""}Ignoring a non-CA certificate that was
 passed as part of the chain.
 WARNING
     } @$certchain;
-    fail("no remaining certificates in chain"), return undef
-        if ! @certchain;
+
+    do {
+        $context->fail_and_release(
+            $testname,
+            "no remaining certificates in chain");
+        return
+     } if ! @certchain;
 
     my $bundlefile = File::Spec->catfile
         (_tempdir(), sprintf("ca-bundle-%d-%d.crt", $$,
                              _unique_number()));
     File::Slurp::write_file($bundlefile,
                             join("\n", @certchain));
+    $context->release();
     return scalar run_thru_openssl($cert, qw(verify),
                                    -CAfile => $bundlefile);
 }
@@ -500,11 +575,17 @@ doesn't contain any B<valid> CA certificate.
 sub certificate_chain_invalid_ok {
     my ($cert, $certchain, $testname) = @_;
 
-    test (($testname || "certificate_chain_ok") => sub {
-        my $out = _run_openssl_verify($cert, $certchain, $testname);
-        return if ! defined $out; # Already failed
-        like($out, qr/error/, "verify failed as expected");
-    });
+    my $context = context();
+    my $out = _run_openssl_verify($cert, $certchain, $testname);
+    do { $context->release;  return } if ! defined $out; # Failed on previous line
+
+    if ($out =~ m/error/) {
+        return $context->pass_and_release($testname);
+    } else {
+        return $context->fail_and_release($testname,
+                                          'unexpected success of `openssl verify`',
+                                         $out);
+    }
 }
 
 =head2 x509_schema ()
@@ -1157,15 +1238,23 @@ if using the following C<openssl> command:
     -extensions usr_cert
 
 where 10958 stands for a validity period of 30 years, so that these
-self-signed certificates seldom actually expire.  Because the default
-configuration is used, the world-famous yet Belgian I<Internet Widgits
-Pty Ltd> company is put in charge as issuer and subject of these
-certificates.
+self-signed certificates seldom actually expire.
+
+The valus in this hash are actually objects, where the certificate in
+pem format is e.g. C<< $test_self_signed_certs{rsa1024}->{pem} >>.
+Other keys are available to tests, such as
+C<<$test_self_signed_certs{rsa1024}->{serial} >> (in the format that
+L<Crypt::OpenSSL::CA/get_serial> outputs).
+
+Because OpenSSL's default configuration was used, the world-famous yet
+Belgian I<Internet Widgits Pty Ltd> company is put in charge as issuer
+and subject of these certificates.
 
 =cut
 
 our %test_self_signed_certs =
-    (rsa1024 => <<"RSA1024",
+    (rsa1024 => {
+        pem => <<"RSA1024",
 -----BEGIN CERTIFICATE-----
 MIICgzCCAeygAwIBAgIJAPecvJ1g5yDDMA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
 BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
@@ -1183,7 +1272,10 @@ e9WIdiVrEIk9kvp4cgnwCF0O/K02/BIpq5MlqSXwGQhQ/o29J4/A4/LobcLDYr11
 mGZJJpjA9oDx7sZF6FbTTa5E+tXZRls=
 -----END CERTIFICATE-----
 RSA1024
-     rsa2048 => <<"RSA2048",
+        serial => "0xf79cbc9d60e720c3",
+    },
+     rsa2048 => {
+         pem => <<"RSA2048",
 -----BEGIN CERTIFICATE-----
 MIIDiDCCAnCgAwIBAgIJAL6sAb2vcVpUMA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
 BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
@@ -1206,6 +1298,8 @@ abDMwiOqndnPFfSNFTWue9PcgpMoT3V+eq6VN0Q6AyPZxkfzVg+VUISli0sXNMKB
 KjI6FX0+FXEYyhmsnkAq83kVYop/ietw/mvJkF1xxpkv/urU2AagNVmaxuo=
 -----END CERTIFICATE-----
 RSA2048
+         serial => "0xbeac01bdaf715a54",
+     },
 );
 
 =head2 %test_rootca_certs
@@ -1222,7 +1316,8 @@ valid certification chain as per RFC3280 section 6.1.4, item k.
 =cut
 
 our %test_rootca_certs =
-    (rsa1024 => <<RSA1024,
+    (rsa1024 => {
+        pem => <<RSA1024,
 -----BEGIN CERTIFICATE-----
 MIICsDCCAhmgAwIBAgIJANdqtXzdPS/1MA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
 BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
@@ -1241,7 +1336,10 @@ yP4YeUHO6FIHd0RyGEnM3cqcoqg8TewXlUwOkHphCrZ5eFbxxEarVz1wwkZqd5z0
 3IInE3EJ7D8rxfbC1c1fdeh8akI=
 -----END CERTIFICATE-----
 RSA1024
-     rsa2048 => <<RSA2048,
+        serial => "0xd76ab57cdd3d2ff",
+    },
+     rsa2048 => {
+         pem => <<RSA2048,
 -----BEGIN CERTIFICATE-----
 MIIDtTCCAp2gAwIBAgIJAJcq/w5w2Nr3MA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
 BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
@@ -1265,6 +1363,8 @@ V1K1Plj27fEC57i71qygF+UogYd802DM/5F76OHKN4CAq54UFSzfzpcjOsYZLCG7
 tKVO4zFWPkXPhdh7brNIn19ayoyESq59WuZhPwZkzOZgaFrHeQA2Dks=
 -----END CERTIFICATE-----
 RSA2048
+         serial => "0x972aff0e70d8daf7"
+     },
 );
 
 =head2 %test_entity_certs
@@ -1279,14 +1379,15 @@ precise C<openssl> commands used are
     -key test.key | \
   openssl ca -batch -days 10958 -in /dev/stdin
 
+The structure of keys in C<%test_entity_certs> is the same as above, i.e. C<< ->{pem} >>
 In particular this means that entries keyed off the same identifier in
 %test_entity_certs and %test_rootca_certs form a valid RFC3280
 certification path: that is,
 
 =for My::Tests::Below "certificate_chain_ok" begin
 
-  certificate_chain_ok($test_entity_certs{$id},
-                       [ $test_rootca_certs{$id} ]);     # Works
+  certificate_chain_ok($test_entity_certs{$id}->{pem},
+                       [ $test_rootca_certs{$id}->{pem} ]);     # Works
 
 =for My::Tests::Below "certificate_chain_ok" end
 
@@ -1294,8 +1395,8 @@ holds for every valid $id.  But conversely,
 
 =for My::Tests::Below "certificate_chain_notok" begin
 
-  certificate_chain_ok($test_entity_certs{$id},
-                       [ $test_self_signed_certs{$id} ]);     # NOT OK!
+  certificate_chain_ok($test_entity_certs{$id}->{pem},
+                       [ $test_self_signed_certs{$id}->{pem} ]);     # NOT OK!
 
 =for My::Tests::Below "certificate_chain_notok" end
 
@@ -1312,7 +1413,8 @@ have the same private key.
 =cut
 
 our %test_entity_certs =
-    (rsa1024 => <<RSA1024,
+    (rsa1024 => {
+        pem => <<RSA1024,
 -----BEGIN CERTIFICATE-----
 MIICjjCCAfegAwIBAgIBCTANBgkqhkiG9w0BAQUFADBFMQswCQYDVQQGEwJBVTET
 MBEGA1UECBMKU29tZS1TdGF0ZTEhMB8GA1UEChMYSW50ZXJuZXQgV2lkZ2l0cyBQ
@@ -1330,7 +1432,10 @@ isSmjCE7Z9D+9i2SQxzcvrG05gLQRAqS1bRAIHfcoBuGS5B+PvbNhzPTly24NvVp
 HSsnQD5qXQq4V/p1hq9OoeCpiQBgOa5wODkpNubGe7k3wQ==
 -----END CERTIFICATE-----
 RSA1024
-     rsa2048 => <<RSA2048,
+        serial => "0x9",
+    },
+     rsa2048 => {
+         pem => <<RSA2048,
 -----BEGIN CERTIFICATE-----
 MIIDkzCCAnugAwIBAgIBCjANBgkqhkiG9w0BAQUFADBFMQswCQYDVQQGEwJBVTET
 MBEGA1UECBMKU29tZS1TdGF0ZTEhMB8GA1UEChMYSW50ZXJuZXQgV2lkZ2l0cyBQ
@@ -1354,7 +1459,9 @@ JjghMlFGnm/wZG5zD1adjIvw66GqKsxPWml2Tc3azZMpobKYCS91h6WT19kZIC5Y
 valACunSZw==
 -----END CERTIFICATE-----
 RSA2048
-     );
+         serial => "0xa",
+     }
+    );
 
 =head1 INTERNAL METHODS
 
@@ -1403,8 +1510,7 @@ __END__
 
 =cut
 
-use Test::More "no_plan";
-use Test::Group;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test;
 
 =head2 Fixture Tests
@@ -1413,10 +1519,10 @@ use Crypt::OpenSSL::CA::Test;
 
 =cut
 
-test "run_thru_openssl" => sub {
+subtest "run_thru_openssl" => sub {
 	my $version = run_thru_openssl(undef, "version");
     is($?, 0);
-    like($version, qr/openssl/i);
+    like($version, qr/(open|libre)ssl/i);
     unlike($version, qr/uninitialized/); # In case there is some barfage
     # going on in the forked Perls...
 
@@ -1435,7 +1541,7 @@ test "run_thru_openssl" => sub {
     isnt($?, 0);
 };
 
-test "run_perl and run_perl_ok" => sub {
+subtest "run_perl and run_perl_ok" => sub {
     my $out;
     run_perl_ok(<<"SCRIPT_OK", \$out);
 print "hello"; # STDOUT
@@ -1446,7 +1552,7 @@ SCRIPT_OK
     my $tempdir = My::Tests::Below->tempdir;
 
     $out = run_perl(<<"SCRIPT_WRAPPER");
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test qw(run_perl_ok);
 
 run_perl_ok <<'SCRIPT_OK';
@@ -1458,7 +1564,8 @@ run_perl_ok <<'SCRIPT_NOT_OK';
 die "argl";
 SCRIPT_NOT_OK
 
-exit(1);
+done_testing;
+
 SCRIPT_WRAPPER
     isnt($?, 0, "run_perl: that script shall exit with nonzero status");
     like($out, qr/not ok 2/m);
@@ -1471,16 +1578,18 @@ SCRIPT_WRAPPER
 
 };
 
-test "errstack_empty_ok" => sub {
+subtest "errstack_empty_ok" => sub {
     errstack_empty_ok();
     my $out = run_perl(<<"SCRIPT_NOT_OK");
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test qw(errstack_empty_ok);
 use Net::SSLeay;
 
 is(Net::SSLeay::BIO_new_file("/no/such/file_", "r"), 0); # OK
 errstack_empty_ok(); # not OK
 
+done_testing;
+
 SCRIPT_NOT_OK
 
     like($out, qr/^ok 1/m);
@@ -1492,7 +1601,7 @@ SCRIPT_NOT_OK
            "errors are reported at the proper stack depth");
 };
 
-test "certificate_looks_ok" => sub {
+subtest "certificate_looks_ok" => sub {
     my $ok_cert = <<'OK_CERT';
 -----BEGIN CERTIFICATE-----
 MIICsDCCAhmgAwIBAgIJAPV18QziY9UvMA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
@@ -1519,7 +1628,7 @@ OK_CERT
     my $out = run_perl(<<"SCRIPT");
 use strict;
 use warnings;
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test qw(certificate_looks_ok);
 
 my \$certificate = <<'OK_CERT';
@@ -1544,6 +1653,9 @@ certificate_looks_ok(\$certificate, "REGRESSION: dud cert");
          # expecting not OK, lest REGRESSION
 
 certificate_looks_ok({}, "Should have thrown (bad input)"); # Should throw
+
+done_testing;
+
 SCRIPT
 
     like($out, qr/^ok 1/m);
@@ -1558,16 +1670,16 @@ SCRIPT
 
 =cut
 
-begin_skipping_tests unless
-    eval { require Devel::Leak; require Devel::Mallinfo; };
-test "no leak" => sub {
+subtest "no leak" => sub {
+    skip_all "Cannot check leaks" unless
+      eval { require Devel::Leak; require Devel::Mallinfo; };
     leaks_SVs_ok { };
     leaks_bytes_ok { };
 };
 
-test "leaking scalars" => sub {
+subtest "leaking scalars" => sub {
     my $leakyscript = <<'LEAKYSCRIPT';
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test;
 sub leak {
    for (1..20) {
@@ -1579,6 +1691,8 @@ sub leak {
 
 leaks_bytes_ok { leak };
 leaks_SVs_ok { leak };
+
+done_testing;
 LEAKYSCRIPT
 
     my $out = run_perl($leakyscript);
@@ -1590,19 +1704,26 @@ LEAKYSCRIPT
            "errors are reported at the proper stack depth");
 };
 
-skip_next_test if cannot_check_bytes_leaks; # Eg MacOS
-test "leaking bytes" => sub {
+subtest "leaking bytes" => sub {
+    skip_all "Cannot check bytes leaks" if cannot_check_bytes_leaks; # Eg MacOS
     my $leakyscript = <<'LEAKYSCRIPT';
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test;
 sub leak {
     # Perl is getting smarter and smarter in memory mgmt, and we need
     # to ward off constant folding for that test to fail as expected:
-    push(our @a, "abcde" x (10000 + scalar @a));
+    my $ballast = "abcde";
+    for(my $i = 0; $i < 15; $i++) {
+       $ballast .= $ballast;
+    }
+    push(our @a, $ballast);
 }
 
 leaks_bytes_ok { leak() };
 leaks_SVs_ok { leak() };
+
+done_testing;
+
 LEAKYSCRIPT
 
     my $out = run_perl($leakyscript);
@@ -1613,7 +1734,7 @@ LEAKYSCRIPT
            "errors are reported at the proper stack depth");
 };
 
-my $cert_pem = $Crypt::OpenSSL::CA::Test::test_self_signed_certs{"rsa1024"};
+my $cert_pem = $Crypt::OpenSSL::CA::Test::test_self_signed_certs{"rsa1024"}->{pem};
 
 # REFACTORME into Crypt::OpenSSL::CA::Test::pem2der or something
 my $cert_der = do {
@@ -1624,7 +1745,7 @@ my $cert_der = do {
     MIME::Base64::decode_base64($_);
 };
 
-test "x509_decoder" => sub {
+subtest "x509_decoder" => sub {
     use MIME::Base64;
     my $decoder = Crypt::OpenSSL::CA::Test::x509_decoder('Certificate');
     ok($decoder->can("decode"));
@@ -1639,14 +1760,13 @@ test "x509_decoder" => sub {
 
 =cut
 
-test "synopsis" => sub {
-    # Thank you Test::Group for being fully reflexive!
+subtest "synopsis" => sub {
     eval My::Tests::Below->pod_code_snippet("synopsis");
     die $@ if $@;
 };
 
 
-test "synopsis asn1" => sub {
+subtest "synopsis asn1" => sub {
     my $synopsis = My::Tests::Below->pod_code_snippet("synopsis-asn1");
     ok(defined(my $dn_der =
        $Crypt::OpenSSL::CA::Test::test_der_DNs{"CN=Zoinx,C=fr"}),
@@ -1659,7 +1779,7 @@ test "synopsis asn1" => sub {
 
 =cut
 
-test "test_simple_utf8 and test_bmp_utf8" => sub {
+subtest "test_simple_utf8 and test_bmp_utf8" => sub {
     is(length(Crypt::OpenSSL::CA::Test->test_simple_utf8()), 6);
     ok(utf8::is_utf8(Crypt::OpenSSL::CA::Test->test_simple_utf8()));
 
@@ -1667,11 +1787,10 @@ test "test_simple_utf8 and test_bmp_utf8" => sub {
     ok(utf8::is_utf8(Crypt::OpenSSL::CA::Test->test_bmp_utf8()));
 };
 
-test "%test_keys_plaintext and %test_keys_password" => sub {
-    is_deeply
-        ([sort keys %Crypt::OpenSSL::CA::Test::test_keys_plaintext],
-         [sort keys %Crypt::OpenSSL::CA::Test::test_keys_password],
-         "same keys in both");
+subtest "%test_keys_plaintext and %test_keys_password" => sub {
+    is([sort keys %Crypt::OpenSSL::CA::Test::test_keys_plaintext],
+       [sort keys %Crypt::OpenSSL::CA::Test::test_keys_password],
+       "same keys in both");
     if (defined(my $openssl_bin = openssl_path)) {
         while(my ($k, $v) =
               each %Crypt::OpenSSL::CA::Test::test_keys_password) {
@@ -1683,25 +1802,27 @@ test "%test_keys_plaintext and %test_keys_password" => sub {
     }
 };
 
-test "certificate_chain_ok and test certificates" => sub {
+subtest "certificate_chain_ok and test certificates" => sub {
     my @keyids = keys %Crypt::OpenSSL::CA::Test::test_rootca_certs;
     foreach my $id (@keyids) {
         certificate_chain_ok
-            ($Crypt::OpenSSL::CA::Test::test_entity_certs{$id},
-           [ $Crypt::OpenSSL::CA::Test::test_rootca_certs{$id} ]);
+            ($Crypt::OpenSSL::CA::Test::test_entity_certs{$id}->{pem},
+           [ $Crypt::OpenSSL::CA::Test::test_rootca_certs{$id}->{pem} ]);
     }
 
     my ($snippet_ok, $snippet_not_ok) =
         map { My::Tests::Below->pod_code_snippet($_) }
             (qw(certificate_chain_ok certificate_chain_notok));
     my $out = run_perl(<<"SCRIPT");
-use Test::More no_plan => 1;
+use Test2::V0;
 use Crypt::OpenSSL::CA::Test qw(certificate_chain_ok
        %test_rootca_certs %test_self_signed_certs %test_entity_certs);
 foreach my \$id (qw(${\join(" ", @keyids)})) {
     $snippet_ok
     $snippet_not_ok
 }
+
+done_testing;
 SCRIPT
     for my $i (0..$#keyids) {
         my $success = 2 * $i + 1;
@@ -1710,3 +1831,5 @@ SCRIPT
         like($out, qr/^not ok $failure/m);
     }
 };
+
+done_testing;
